#ifndef PARTITION_HPP
#define PARTITION_HPP

#include <map>
#include <vector>
#include <gp-bnb/graph.hpp>

using vertex_id = unsigned int;

// Represents potential solutions
class partition {
public:
    enum subgraph { sg_a = -1, sg_b = 1, none = 0 };
    /** @brief  Initializes an empty partition
     *  @param  Pointer to the graph to be partitioned
     */
    partition(graph* g);
    /** @brief  Returns the number of vertices of an element of the partition
     *  @param  Subgraph A, B or none
     *  @return Number of vertices of sg
     */
    unsigned int num_vertices(subgraph sg);
    /** @brief  Assigns a vertex to a subgraph
     *  @param  Vertex Id
     *  @param  Subgraph A or B
     */
    void assign_vertex(vertex_id v, subgraph sg); 
    /** @brief  Reverts the assignment of a vertex to a subgraph
     *  @param  Vertex Id
     */
    void unassign_vertex(vertex_id v);

private:
    graph* supergraph_;
    std::vector<subgraph> vertex_assignments_;
    std::map<subgraph,unsigned int> vertices_;
    unsigned int current_objective_ = 0;
};

#endif
